"""Cross-platform file path widget.

Uses QgsFileWidget when running inside QGIS, falls back to a
QLineEdit + browse button for standalone mode.
"""

try:
    from qgis.gui import QgsFileWidget
except ImportError:
    QgsFileWidget = None

from qgis.PyQt.QtWidgets import (
    QFileDialog,
    QHBoxLayout,
    QLineEdit,
    QToolButton,
    QWidget,
)


class FilePathWidget(QWidget):
    """A file path selector widget that works in both QGIS and standalone."""

    class StorageMode:
        GetFile = "get_file"
        SaveFile = "save_file"
        GetDirectory = "get_directory"

    def __init__(self, parent=None, storage_mode=None, filter_string=""):
        super().__init__(parent)
        self.__filter = filter_string
        self.__storage_mode = storage_mode or self.StorageMode.GetFile

        if QgsFileWidget is not None:
            self.__qgs_widget = QgsFileWidget(self)
            qgs_mode = {
                self.StorageMode.GetFile: QgsFileWidget.StorageMode.GetFile,
                self.StorageMode.SaveFile: QgsFileWidget.StorageMode.SaveFile,
                self.StorageMode.GetDirectory: QgsFileWidget.StorageMode.GetDirectory,
            }[self.__storage_mode]
            self.__qgs_widget.setStorageMode(qgs_mode)
            if filter_string:
                self.__qgs_widget.setFilter(filter_string)
            layout = QHBoxLayout(self)
            layout.setContentsMargins(0, 0, 0, 0)
            layout.addWidget(self.__qgs_widget)
        else:
            self.__qgs_widget = None
            layout = QHBoxLayout(self)
            layout.setContentsMargins(0, 0, 0, 0)
            self.__line_edit = QLineEdit(self)
            self.__browse_button = QToolButton(self)
            self.__browse_button.setText("…")
            self.__browse_button.clicked.connect(self.__browse)
            layout.addWidget(self.__line_edit)
            layout.addWidget(self.__browse_button)

    def filePath(self) -> str:
        if self.__qgs_widget is not None:
            return self.__qgs_widget.filePath()
        return self.__line_edit.text()

    def setFilePath(self, path: str):
        if self.__qgs_widget is not None:
            self.__qgs_widget.setFilePath(path)
        else:
            self.__line_edit.setText(path)

    def __browse(self):
        if self.__storage_mode == self.StorageMode.SaveFile:
            path, _ = QFileDialog.getSaveFileName(self, self.tr("Select file"), "", self.__filter)
        elif self.__storage_mode == self.StorageMode.GetDirectory:
            path = QFileDialog.getExistingDirectory(self, self.tr("Select directory"))
        else:
            path, _ = QFileDialog.getOpenFileName(self, self.tr("Select file"), "", self.__filter)
        if path:
            self.__line_edit.setText(path)



# -----------------------------------------------------------
#
# Profile
# Copyright (C) 2025  Denis Rouzaud
# -----------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, print to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# ---------------------------------------------------------------------

from qgis.PyQt.QtCore import Qt
from qgis.PyQt.QtWidgets import QDialog, QVBoxLayout

from ..libs.pgserviceparser.gui.message_bar import MessageBar
from ..libs.pum.dumper import Dumper
from ..utils.pg_utils import find_pg_executable
from ..utils.plugin_utils import PluginUtils, logger
from ..utils.qt_utils import OverrideCursor
from .file_path_widget import FilePathWidget

DIALOG_UI = PluginUtils.get_ui_class("database_restore_dialog.ui")


class DatabaseRestoreDialog(QDialog, DIALOG_UI):

    _session_defaults = {
        "file_path": "",
        "exclude_schemas": "",
    }

    def __init__(self, service_name, parent=None):
        QDialog.__init__(self, parent)
        self.setupUi(self)

        self.__service_name = service_name

        # Message bar
        self.__message_bar = MessageBar(self)
        placeholder_layout = QVBoxLayout(self.messageBar_placeholder)
        placeholder_layout.setContentsMargins(0, 0, 0, 0)
        placeholder_layout.addWidget(self.__message_bar)

        # File path widget
        self.__file_path_widget = FilePathWidget(
            parent=self,
            storage_mode=FilePathWidget.StorageMode.GetFile,
            filter_string=self.tr("Dump files (*.dump *.backup *.sql);;All files (*)"),
        )
        file_placeholder_layout = QVBoxLayout(self.filePath_placeholder)
        file_placeholder_layout.setContentsMargins(0, 0, 0, 0)
        file_placeholder_layout.addWidget(self.__file_path_widget)

        # Restore session defaults
        self.__file_path_widget.setFilePath(self._session_defaults["file_path"])
        self.excludeSchemas_lineEdit.setText(self._session_defaults["exclude_schemas"])

        self.buttonBox.accepted.connect(self._accept)

    def _accept(self):
        file_path = self.__file_path_widget.filePath().strip()
        if not file_path:
            self.__message_bar.pushError(self.tr("Please select a dump file."))
            return

        exclude_schemas = [
            s.strip() for s in self.excludeSchemas_lineEdit.text().split(",") if s.strip()
        ]

        try:
            with OverrideCursor(Qt.CursorShape.WaitCursor):
                dumper = Dumper(
                    pg_connection=self.__service_name,
                    dump_path=file_path,
                )
                dumper.pg_restore(
                    pg_restore_exe=find_pg_executable("pg_restore"),
                    exclude_schema=exclude_schemas or None,
                )
        except Exception as e:
            errorText = self.tr(f"Error restoring database:\n{e}")
            logger.error(errorText)
            self.__message_bar.pushError(errorText)
            return

        # Save session defaults
        DatabaseRestoreDialog._session_defaults["file_path"] = file_path
        DatabaseRestoreDialog._session_defaults["exclude_schemas"] = (
            self.excludeSchemas_lineEdit.text()
        )

        super().accept()
